unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls;

type
  TpdScriptFunc = function(ScriptFile:PWideChar; Params:PWideChar; SyntaxCheckOnly:Bool): Integer; stdcall;
  TpdScriptExFunc = function(ScriptFile:PWideChar; Params:PWideChar; SyntaxCheckOnly:Bool; out pStdOut: PWideChar): Integer; stdcall;
  TpdScriptExLibFunc = function(ScriptFile:PWideChar; Params:PWideChar; SyntaxCheckOnly:BOOL; out pStdOut: PWideChar; LibraryPath:PWideChar): Integer; stdcall;
  TpdScriptInlineExFunc = function(ScriptCode:PWideChar; ScriptDFM: PWideChar; Params:PWideChar; SyntaxCheckOnly:Bool; out pStdOut: PWideChar): Integer; stdcall;

  TpdsFreeMem = procedure(p: Pointer); stdcall;
  TpdsFreeScript = procedure(pScript: THandle); stdcall;

  TpdsExecuteFunc = function(pScript: THandle; FncName: PWideChar; Params: PWideChar; PrmDelim:PWideChar): Integer; stdcall;
  TpdsExecuteFuncEx = function(pScript: THandle; FncName: PWideChar; Params: PWideChar; PrmDelim: PWideChar; VarParams: BOOL; out FncResult:PWideChar): Integer; stdcall;

  TForm3 = class(TForm)
    Label1: TLabel;
    lbLibFile: TLabel;
    Label2: TLabel;
    mReport: TMemo;
    Label3: TLabel;
    Label4: TLabel;
    lbExitCode: TLabel;
    lbLibCLFile: TLabel;
    pcMain: TPageControl;
    tsSimple: TTabSheet;
    tsInline: TTabSheet;
    tsCallback: TTabSheet;
    sbConsole: TButton;
    sbClock: TButton;
    sbTwoForms: TButton;
    mS1: TMemo;
    sbDirectInt: TButton;
    sbSCallback: TButton;
    mSCallback: TMemo;
    Label5: TLabel;
    sbAnyScript: TButton;
    tsKeepAlive: TTabSheet;
    sbClockNonModal: TButton;
    sbClockFree: TButton;
    sbClockInside: TButton;
    pClockInside: TPanel;
    sbClockInsideChange: TButton;
    Label6: TLabel;
    sbCallScriptFunc: TButton;
    procedure sbConsoleClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sbDirectIntClick(Sender: TObject);
    procedure sbClockClick(Sender: TObject);
    procedure sbTwoFormsClick(Sender: TObject);
    procedure sbSCallbackClick(Sender: TObject);
    procedure sbAnyScriptClick(Sender: TObject);
    procedure sbClockNonModalClick(Sender: TObject);
    procedure sbClockFreeClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure sbClockInsideClick(Sender: TObject);
    procedure sbClockInsideChangeClick(Sender: TObject);
    procedure sbCallScriptFuncClick(Sender: TObject);
  private
    { Private declarations }
    LibFileName:string;
    LibFileNameCL:string;
    LibHandle:THandle;
    LibHandleCL:THandle;

    pdsFreeScript:TpdsFreeScript;
    pScriptList:TList; // holds list of not finished scripts (with keep-alive flag)
    pScriptInside:THandle; // temporary pointer to last script plugged into pClockInside panel
  public
    { Public declarations }
  end;

var
  Form3: TForm3;

implementation

{$R *.dfm}

procedure TForm3.FormCreate(Sender: TObject);
var
  tmp:string;
  tmpCL:string;
begin
  LibFileName:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\..\..\pdScriptE.dll');
  if not fileexists(LibFileName) then
  begin
    LibFileName:=ExtractFilePath(ParamStr(0))+'pdScriptE.dll';
    if not fileexists(LibFileName) then
      LibFileName:='pdScriptE.dll';
  end;
  LibFileNameCL:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\..\..\pdScriptECL.dll');
  if not fileexists(LibFileNameCL) then
  begin
    LibFileNameCL:=ExtractFilePath(ParamStr(0))+'pdScriptECL.dll';
    if not fileexists(LibFileNameCL) then
      LibFileNameCL:='pdScriptECL.dll';
  end;

  LibHandle:=LoadLibrary(PChar(LibFileName));
  if LibHandle>0 then
    lbLibFile.Caption:=LibFileName
  else
  begin
    lbLibFile.Caption:='pdScriptE.dll not found!';
    sbClock.Enabled:=False;
  end;

  LibHandleCL:=LoadLibrary(PChar(LibFileNameCL));
  if LibHandleCL>0 then
    lbLibCLFile.Caption:=LibFileNameCL
  else
  begin
    lbLibCLFile.Caption:='pdScriptECL.dll not found!';
    sbConsole.Enabled:=False;
    sbDirectInt.Enabled:=False;
  end;

  pScriptList:=TList.Create;
  pScriptInside:=0;
end;

procedure TForm3.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  while pScriptList.Count>0 do
    sbClockFreeClick(sbClockFree);
end;

procedure TForm3.FormDestroy(Sender: TObject);
begin

  pScriptList.Free;
  if LibHandle>0 then
    FreeLibrary(LibHandle);
  if LibHandleCL>0 then
    FreeLibrary(LibHandleCL);
end;

procedure TForm3.sbClockClick(Sender: TObject);
var
  pdScriptExLib:TpdScriptExLibFunc;
  pdsFreeMem:TpdsFreeMem;
  fn:WideString;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  fn:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\Clock\Clock.dpas');
  try
    if LibHandle>0 then
    begin
      @pdScriptExLib := GetProcAddress(LibHandle,'pdScriptExLib');
      @pdsFreeMem := GetProcAddress(LibHandle,'pdsFreeMem');
      if (@pdScriptExLib=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdScriptExLib(PWideChar(fn),nil,False,rep,nil);
        lbExitCode.Caption:=IntToStr(ec);
        mReport.Lines.Text:=rep;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbConsoleClick(Sender: TObject);
var
  pdScriptEx:TpdScriptExFunc;
  pdsFreeMem:TpdsFreeMem;
  fn:WideString;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  fn:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\Console\ConsoleDemo.rops');
  try
    if LibHandleCL>0 then
    begin
      @pdScriptEx := GetProcAddress(LibHandleCL,'pdScriptEx');
      @pdsFreeMem := GetProcAddress(LibHandleCL,'pdsFreeMem');
      if (@pdScriptEx=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdScriptEx(PWideChar(fn),nil,True,rep);
        lbExitCode.Caption:=IntToStr(ec);
        mReport.Lines.Text:=rep;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbDirectIntClick(Sender: TObject);
var
  pdScriptInlineEx:TpdScriptInlineExFunc;
  pdsFreeMem:TpdsFreeMem;
  code:WideString;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  try
    if LibHandleCL>0 then
    begin
      @pdScriptInlineEx := GetProcAddress(LibHandleCL,'pdScriptInlineEx');
      @pdsFreeMem := GetProcAddress(LibHandleCL,'pdsFreeMem');
      if (@pdScriptInlineEx=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        code:=mS1.Lines.Text;
        ec:=pdScriptInlineEx(PWideChar(code),nil,nil,False,rep);
        lbExitCode.Caption:=IntToStr(ec);
        mReport.Lines.Text:=rep;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

{function MyPDScriptCallback(cCommand:PChar; cParams:PChar; cBuffer:PChar; cBufSize:Integer):Integer; stdcall;
begin
  Result:=0;
  if AnsiSameText(cCommand,'GetCaption') then
  begin
    if (cBuffer=nil) or (cBufSize<=0) then
      Result:=Length(Form3.Caption)
    else
      StrPLCopy(cBuffer,Form3.Caption,cBufSize);
  end
  else
  if AnsiSameText(cCommand,'SetCaption') then
  begin
    Form3.Caption:=cParams;
  end;
end;}

function MyPDScriptCallback(cCommand:PChar; cParams:PChar; cBuffer:PChar; cBufSize:Integer):Integer; stdcall;
begin
  Result:=0;
  if AnsiSameText(cCommand,'GetCaption') then
  begin
    if (cBuffer=nil) or (cBufSize<=0) then
      Result:=Length(Form3.Caption)
    else
      StrPLCopy(cBuffer,Form3.Caption,cBufSize);
  end
  else
  if AnsiSameText(cCommand,'SetCaption') then
  begin
    Form3.Caption:=cParams;
  end;
end;

procedure TForm3.sbSCallbackClick(Sender: TObject);
var
  pdScriptInlineEx:TpdScriptInlineExFunc;
  pdsFreeMem:TpdsFreeMem;
  code:WideString;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  try
    if LibHandle>0 then
    begin
      @pdScriptInlineEx := GetProcAddress(LibHandle,'pdScriptInlineEx');
      @pdsFreeMem := GetProcAddress(LibHandle,'pdsFreeMem');
      if (@pdScriptInlineEx=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        code:=mSCallback.Lines.Text;
        ec:=pdScriptInlineEx(PWideChar(code),nil,PWideChar('"/pds{host-callback='+IntToStr(Int64(@MyPDScriptCallback))+'}"'),False,rep);
        {
          // deprecated variant of pdScript options parameter:
          ec:=pdScriptInlineEx(PWideChar(code),nil,PWideChar('/PDSH'+IntToStr(Int64(@MyPDScriptCallback))),False,rep);
        }
        lbExitCode.Caption:=IntToStr(ec);
        mReport.Lines.Text:=rep;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbTwoFormsClick(Sender: TObject);
var
  pdScriptExLib:TpdScriptExLibFunc;
  pdsFreeMem:TpdsFreeMem;
  fn:WideString;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  fn:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\Two Forms\TwoForms.dpas');
  try
    if LibHandle>0 then
    begin
      @pdScriptExLib := GetProcAddress(LibHandle,'pdScriptExLib');
      @pdsFreeMem := GetProcAddress(LibHandle,'pdsFreeMem');
      if (@pdScriptExLib=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdScriptExLib(PWideChar(fn),PWideChar('"/pds{hide-from-taskbar=true}"'),False,rep,nil);
        lbExitCode.Caption:=IntToStr(ec);
        mReport.Lines.Text:=rep;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbAnyScriptClick(Sender: TObject);
var
  pdScriptExLib:TpdScriptExLibFunc;
  pdsFreeMem:TpdsFreeMem;
  fn:WideString;
  d,f:string;
  rep:PWideChar;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  d:=ExtractFilePath(ExpandFileName(ExtractFilePath(ParamStr(0))+'..\dummy.dpas'));
  if PromptForFileName(f,'Script files (*.dpas, *.pas, *.rops)|*.dpas;*.pas;*.rops|All files (*.*)|*.*','dpas',sbAnyScript.Caption,d) then
  begin
    try
      fn:=f;
      if LibHandle>0 then
      begin
        @pdScriptExLib := GetProcAddress(LibHandle,'pdScriptExLib');
        @pdsFreeMem := GetProcAddress(LibHandle,'pdsFreeMem');
        if (@pdScriptExLib=nil) or (@pdsFreeMem=nil) then
          RaiseLastWin32Error
        else
        begin
          ec:=pdScriptExLib(PWideChar(fn),nil,False,rep,nil);
          lbExitCode.Caption:=IntToStr(ec);
          mReport.Lines.Text:=rep;
          pdsFreeMem(rep);
        end;
      end;
    except
      on E:Exception do
        ShowMessage(e.message);
    end;
  end;
end;

procedure TForm3.sbClockNonModalClick(Sender: TObject);
var
  pdScript:TpdScriptFunc;
  fn:WideString;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  fn:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\Clock\Clock.dpas');
  try
    if LibHandle>0 then
    begin
      @pdScript := GetProcAddress(LibHandle,'pdScript');
      @pdsFreeScript := GetProcAddress(LibHandle,'pdsFreeScript');
      if (@pdScript=nil) or (@pdsFreeScript=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdScript(PWideChar(fn),PWideChar('"/pds{keep-alive=true;host-app='+IntToStr(Self.Handle)+'}"'),False);
        if ec<=1 then // returned pdScript interpreter instance must have the handle greater than 1
          lbExitCode.Caption:=IntToStr(ec)
        else
          pScriptList.Add(Pointer(ec));
        sbClockFree.Enabled:=pScriptList.Count>0;
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbClockInsideClick(Sender: TObject);
var
  pdScript:TpdScriptFunc;
  fn:WideString;
  ec:Integer;
begin
  mReport.Lines.Clear;
  lbExitCode.Caption:='';
  fn:=ExtractFilePath(ParamStr(0))+'\ClockAsFrame.dpas';
  try
    if LibHandle>0 then
    begin
      @pdScript := GetProcAddress(LibHandle,'pdScript');
      @pdsFreeScript := GetProcAddress(LibHandle,'pdsFreeScript');
      if (@pdScript=nil) or (@pdsFreeScript=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdScript(PWideChar(fn),PWideChar('"/pds{keep-alive=true;host-app='+IntToStr(Self.Handle)+
          ';host-parent='+IntToStr(pClockInside.Handle)+';host-bounds=8,8}"'),False);
        if ec<=1 then // returned pdScript interpreter instance must have the handle greater than 1
          lbExitCode.Caption:=IntToStr(ec)
        else
        begin
          pScriptInside:=ec;
          pScriptList.Add(Pointer(ec));
        end;
        sbClockFree.Enabled:=pScriptList.Count>0;
        sbClockInsideChange.Enabled:=sbClockFree.Enabled;
        sbCallScriptFunc.Enabled:=sbClockFree.Enabled;
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbClockInsideChangeClick(Sender: TObject);
var
  pdsExecFunc:TpdsExecuteFunc;
  ec:Integer;
begin
  try
    if (LibHandle>0) and (pScriptInside<>0) then
    begin
      @pdsExecFunc := GetProcAddress(LibHandle,'pdsExecuteFunction');
      if (@pdsExecFunc=nil) then
        RaiseLastWin32Error
      else
      begin
        ec:=pdsExecFunc(pScriptInside,'ChangeFields','Changed text 1;Changed text 2',';');
        if ec<>0 then // return value is always -2 or -1 on error (and 0 on ok), so if you call function with result value, you can freely rely on it
          lbExitCode.Caption:=IntToStr(ec);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbCallScriptFuncClick(Sender: TObject);
var
  pdsExecFuncEx:TpdsExecuteFuncEx;
  pdsFreeMem:TpdsFreeMem;
  ec:Integer;
  rep:PWideChar;

  StrTime:string;
  CurrTime:TDateTime;
  DiffTime:TDateTime;
begin
  try
    if (LibHandle>0) and (pScriptInside<>0) then
    begin
      @pdsExecFuncEx := GetProcAddress(LibHandle,'pdsExecuteFunctionEx');
      @pdsFreeMem := GetProcAddress(LibHandle,'pdsFreeMem');
      if (@pdsExecFuncEx=nil) or (@pdsFreeMem=nil) then
        RaiseLastWin32Error
      else
      begin
        StrTime:='';
        CurrTime:=now;

        ec:=pdsExecFuncEx(pScriptInside,'GetMyTimes',PWideChar(StrTime+';'+FloatToStr(CurrTime)),';',True,rep);
        if ec<>0 then // return value is always -2 or -1 on error (and 0 on ok)
        begin
          lbExitCode.Caption:=IntToStr(ec);
          mReport.Lines.Text:=rep;
        end
        else
        begin
          StrTime:=rep;
          ec:=lastdelimiter(';',StrTime);
          if ec>0 then
          begin
            DiffTime:=StrToFloatDef(Copy(StrTime,ec+1,maxint),0);
            Delete(StrTime,ec,maxint);
            ec:=lastdelimiter(';',StrTime);
            if ec>0 then
            begin
              CurrTime:=StrToFloatDef(Copy(StrTime,ec+1,maxint),0);
              Delete(StrTime,ec,maxint);
            end;
          end;
          mReport.Lines.Text:='Displayed time: '+StrTime+#13#10+
            'Current time: '+TimeToStr(CurrTime)+#13#10+
            'Difference: '+TimeToStr(CurrTime-StrToTimeDef(StrTime,0));
        end;
        pdsFreeMem(rep);
      end;
    end;
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TForm3.sbClockFreeClick(Sender: TObject);
var
  pScript:THandle;
begin
  if pScriptList.Count>0 then
  begin
    if @pdsFreeScript=nil then
    begin
      pScriptList.Clear;
      pScriptInside:=0;
    end
    else
    begin
      pScript:=THandle(pScriptList.Items[0]);
      pScriptList.Delete(0);
      {pScript:=THandle(pScriptList.Items[pScriptList.Count-1]);
      pScriptList.Delete(pScriptList.Count-1);}
      if pScriptInside=pScript then
        pScriptInside:=0;
      pdsFreeScript(pScript);
    end;
  end;
  sbClockFree.Enabled:=pScriptList.Count>0;
  sbClockInsideChange.Enabled:=pScriptInside<>0;
  sbCallScriptFunc.Enabled:=pScriptInside<>0;
end;

end.
